==> CouchCover-0.0.4.tgz: package/lib/main.js <==
(function() {
  var CouchFunction, FilterFunction, UpdateFunction, ViewMapFunction, ViewReduceFunction, createSandbox, findModule, inspect, makeRefStack, makeRequireFun, normalizeFunctionDefinition, puts, readPath, runInNewContext, _ref;
  var __slice = Array.prototype.slice, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  };
  runInNewContext = process.binding('evals').Script.runInNewContext;
  _ref = require('sys'), puts = _ref.puts, inspect = _ref.inspect;
  exports.DesignDoc = (function() {
    function DesignDoc(ddoc) {
      this.ddoc = ddoc;
    }
    DesignDoc.prototype.compile = function(funPath, overrides) {
      if (overrides == null) {
        overrides = {};
      }
      return new CouchFunction(this.ddoc, funPath, overrides);
    };
    DesignDoc.prototype.call = function(funPath, funArgs, overrides) {
      var _ref;
      if (funArgs == null) {
        funArgs = [];
      }
      if (overrides == null) {
        overrides = {};
      }
      return (_ref = this.compile(funPath, overrides)).call.apply(_ref, funArgs);
    };
    DesignDoc.prototype.viewMap = function(viewName, funArgs) {
      var fun, retVal;
      if (funArgs == null) {
        funArgs = [];
      }
      fun = new ViewMapFunction(this.ddoc, viewName);
      retVal = fun.call.apply(fun, funArgs);
      return {
        returned: retVal,
        emitted: fun.emitted
      };
    };
    DesignDoc.prototype.viewReduce = function(viewName, funArgs) {
      var _ref;
      if (funArgs == null) {
        funArgs = [];
      }
      return (_ref = new ViewReduceFunction(this.ddoc, viewName)).call.apply(_ref, funArgs);
    };
    DesignDoc.prototype.update = function() {
      var args, funArgs, updateFun, updateName;
      updateName = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      if (args.length === 1) {
        updateFun = new UpdateFunction(this.ddoc, updateName);
        funArgs = args[0];
      } else {
        updateFun = new UpdateFunction(this.ddoc, updateName, args[0]);
        funArgs = args[1];
      }
      return updateFun.call.apply(updateFun, funArgs);
    };
    DesignDoc.prototype.filter = function(filterName, funArgs) {
      var _ref;
      if (funArgs == null) {
        funArgs = [];
      }
      return (_ref = new FilterFunction(this.ddoc, filterName)).call.apply(_ref, funArgs);
    };
    DesignDoc.prototype.require = function(moduleID) {
      var code, modulePath, sandbox;
      modulePath = moduleID.replace('/', '.');
      code = readPath(modulePath, this.ddoc);
      sandbox = createSandbox(this.ddoc, modulePath, [], {
        exports: {}
      });
      runInNewContext(code, sandbox, "" + this.ddoc._id + "/" + moduleID);
      return sandbox.exports;
    };
    return DesignDoc;
  })();
  CouchFunction = (function() {
    function CouchFunction(ddoc, funPath, overrides) {
      var code, sandbox;
      this.ddoc = ddoc;
      this.funPath = funPath;
      this.overrides = overrides != null ? overrides : {};
      this.log = [];
      this.fileName = "" + this.ddoc._id + "/" + this.funPath + ".js";
      code = normalizeFunctionDefinition(readPath(this.funPath, this.ddoc));
      sandbox = createSandbox(this.ddoc, this.funPath, this.log, this.overrides);
      try {
        this.fun = runInNewContext(code, sandbox, this.fileName);
      } catch (e) {

      }
      if (typeof this.fun !== 'function') {
        throw new NotAFunctionError(this.funPath, this.ddoc);
      }
    }
    CouchFunction.prototype.call = function() {
      var funArgs;
      funArgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return this.fun.apply(null, funArgs);
    };
    return CouchFunction;
  })();
  readPath = function(propPath, obj) {
    var getSubObject;
    getSubObject = function(o, prop) {
      if (typeof o === 'object' && prop in o) {
        return o[prop];
      } else {
        throw new MissingPropPathError(propPath, obj);
      }
    };
    return propPath.split('.').reduce(getSubObject, obj);
  };
  normalizeFunctionDefinition = function(code) {
    var withoutSemicolon;
    withoutSemicolon = code.replace(/;$/, '');
    return "(" + withoutSemicolon + ");";
  };
  createSandbox = function(ddoc, propPath, logEntries, overrides) {
    var fun, name, sb;
    sb = {
      log: function(msg) {
        return logEntries.push(msg);
      },
      require: makeRequireFun(makeRefStack(ddoc, propPath))
    };
    for (name in overrides) {
      fun = overrides[name];
      sb[name] = fun;
    }
    return sb;
  };
  makeRefStack = function(ddoc, funPath) {
    var stack;
    stack = [ddoc];
    funPath.split('.').forEach(function(prop) {
      var currObj;
      currObj = stack[stack.length - 1];
      return stack.push(currObj[prop]);
    });
    stack.pop();
    return stack;
  };
  makeRequireFun = function(refStack) {
    return function(moduleID) {
      var code, nextRefStack, sandbox, _ref;
      sandbox = {
        exports: {}
      };
      try {
        _ref = findModule(moduleID, refStack), nextRefStack = _ref[0], code = _ref[1];
        sandbox.require = makeRequireFun(nextRefStack);
        runInNewContext(code, sandbox, moduleID);
      } catch (e) {
        throw "Design doc require() error: " + e;
      }
      return sandbox.exports;
    };
  };
  findModule = function(moduleID, refStack) {
    var code, current, next, nextRefStack, part, _i, _len, _ref;
    code = null;
    nextRefStack = /^\.\.?\//.test(moduleID) ? refStack.slice() : refStack.slice(0, 1);
    _ref = moduleID.split('/');
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      part = _ref[_i];
      if (part === '..') {
        if (nextRefStack.length > 1) {
          nextRefStack.pop();
        }
      } else if (part !== '.') {
        current = nextRefStack[nextRefStack.length - 1];
        next = current[part];
        if (next == null) {
          throw "Couldn't find part '" + part + "' of '" + moduleID + "'";
        }
        nextRefStack.push(next);
      }
    }
    code = nextRefStack.pop();
    return [nextRefStack, code];
  };
  ViewMapFunction = (function() {
    __extends(ViewMapFunction, CouchFunction);
    function ViewMapFunction(ddoc, viewName) {
      var emitted;
      this.ddoc = ddoc;
      this.viewName = viewName;
      this.emitted = emitted = [];
      ViewMapFunction.__super__.constructor.call(this, this.ddoc, "views." + viewName + ".map", {
        emit: function(k, v) {
          return emitted.push({
            key: k,
            value: v
          });
        }
      });
    }
    return ViewMapFunction;
  })();
  ViewReduceFunction = (function() {
    __extends(ViewReduceFunction, CouchFunction);
    function ViewReduceFunction(ddoc, viewName) {
      this.ddoc = ddoc;
      this.viewName = viewName;
      ViewReduceFunction.__super__.constructor.call(this, this.ddoc, "views." + viewName + ".reduce", {
        sum: function(values) {
          return values.reduce((function(total, n) {
            return total + n;
          }), 0);
        }
      });
    }
    return ViewReduceFunction;
  })();
  UpdateFunction = (function() {
    __extends(UpdateFunction, CouchFunction);
    function UpdateFunction(ddoc, updateName, docID) {
      this.ddoc = ddoc;
      this.updateName = updateName;
      this.docID = docID;
      UpdateFunction.__super__.constructor.call(this, this.ddoc, "updates." + updateName);
    }
    UpdateFunction.prototype.call = function(doc, req) {
      if (doc == null) {
        doc = {};
      }
      if (req == null) {
        req = {};
      }
      if (this.docID != null) {
        req.id = this.docID;
      }
      return UpdateFunction.__super__.call.call(this, doc, req);
    };
    return UpdateFunction;
  })();
  FilterFunction = (function() {
    __extends(FilterFunction, CouchFunction);
    function FilterFunction(ddoc, updateName) {
      this.ddoc = ddoc;
      this.updateName = updateName;
      FilterFunction.__super__.constructor.call(this, this.ddoc, "filters." + updateName);
    }
    return FilterFunction;
  })();
  exports.MissingPropPathError = (function() {
    __extends(MissingPropPathError, Error);
    function MissingPropPathError(funPath, ddoc) {
      this.funPath = funPath;
      this.ddoc = ddoc;
      this.message = "Property path '" + funPath + "' not found in design doc '" + this.ddoc._id + "'";
    }
    return MissingPropPathError;
  })();
  exports.NotAFunctionError = (function() {
    __extends(NotAFunctionError, Error);
    function NotAFunctionError(funPath, ddoc) {
      this.funPath = funPath;
      this.ddoc = ddoc;
      this.message = "Property path '" + funPath + "' in design doc '" + this.ddoc._id + "' does not evaluate to a function";
    }
    return NotAFunctionError;
  })();
}).call(this);

==> CouchCover-0.0.5.tgz: package/lib/main.js <==
(function() {
  var CouchFunction, FilterFunction, MissingPropPathError, NotAFunctionError, UpdateFunction, ViewMapFunction, ViewReduceFunction, createSandbox, findModule, inspect, makeRefStack, makeRequireFun, normalizeFunctionDefinition, puts, readPath, runInNewContext, _ref;
  var __slice = Array.prototype.slice, __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  };
  runInNewContext = process.binding('evals').Script.runInNewContext;
  _ref = require('sys'), puts = _ref.puts, inspect = _ref.inspect;
  exports.DesignDoc = (function() {
    function DesignDoc(ddoc) {
      this.ddoc = ddoc;
    }
    DesignDoc.prototype.compile = function(funPath, overrides) {
      if (overrides == null) {
        overrides = {};
      }
      return new CouchFunction(this.ddoc, funPath, overrides);
    };
    DesignDoc.prototype.call = function(funPath, funArgs, overrides) {
      var _ref2;
      if (funArgs == null) {
        funArgs = [];
      }
      if (overrides == null) {
        overrides = {};
      }
      return (_ref2 = this.compile(funPath, overrides)).call.apply(_ref2, funArgs);
    };
    DesignDoc.prototype.viewMap = function(viewName, funArgs) {
      var fun, retVal;
      if (funArgs == null) {
        funArgs = [];
      }
      fun = new ViewMapFunction(this.ddoc, viewName);
      retVal = fun.call.apply(fun, funArgs);
      return {
        returned: retVal,
        emitted: fun.emitted
      };
    };
    DesignDoc.prototype.viewReduce = function(viewName, funArgs) {
      var _ref2;
      if (funArgs == null) {
        funArgs = [];
      }
      return (_ref2 = new ViewReduceFunction(this.ddoc, viewName)).call.apply(_ref2, funArgs);
    };
    DesignDoc.prototype.update = function() {
      var args, funArgs, updateFun, updateName;
      updateName = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
      if (args.length === 1) {
        updateFun = new UpdateFunction(this.ddoc, updateName);
        funArgs = args[0];
      } else {
        updateFun = new UpdateFunction(this.ddoc, updateName, args[0]);
        funArgs = args[1];
      }
      return updateFun.call.apply(updateFun, funArgs);
    };
    DesignDoc.prototype.filter = function(filterName, funArgs) {
      var _ref2;
      if (funArgs == null) {
        funArgs = [];
      }
      return (_ref2 = new FilterFunction(this.ddoc, filterName)).call.apply(_ref2, funArgs);
    };
    DesignDoc.prototype.require = function(moduleID) {
      var code, modulePath, sandbox;
      modulePath = moduleID.replace('/', '.');
      code = readPath(modulePath, this.ddoc);
      sandbox = createSandbox(this.ddoc, modulePath, [], {
        exports: {}
      });
      runInNewContext(code, sandbox, "" + this.ddoc._id + "/" + moduleID);
      return sandbox.exports;
    };
    return DesignDoc;
  })();
  CouchFunction = (function() {
    function CouchFunction(ddoc, funPath, overrides) {
      var code, sandbox;
      this.ddoc = ddoc;
      this.funPath = funPath;
      this.overrides = overrides != null ? overrides : {};
      this.log = [];
      this.fileName = "" + this.ddoc._id + "/" + this.funPath + ".js";
      code = normalizeFunctionDefinition(readPath(this.funPath, this.ddoc));
      sandbox = createSandbox(this.ddoc, this.funPath, this.log, this.overrides);
      this.fun = (function() {
        try {
          return runInNewContext(code, sandbox, this.fileName);
        } catch (e) {

        }
      }).call(this);
      if (typeof this.fun !== 'function') {
        throw new NotAFunctionError(this.funPath, this.ddoc);
      }
    }
    CouchFunction.prototype.call = function() {
      var funArgs;
      funArgs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      return this.fun.apply(null, funArgs);
    };
    return CouchFunction;
  })();
  readPath = function(propPath, obj) {
    var getSubObject;
    getSubObject = function(o, prop) {
      if (typeof o === 'object' && prop in o) {
        return o[prop];
      } else {
        throw new MissingPropPathError(propPath, obj);
      }
    };
    return propPath.split('.').reduce(getSubObject, obj);
  };
  normalizeFunctionDefinition = function(code) {
    var withoutSemicolon;
    withoutSemicolon = code.replace(/;$/, '');
    return "(" + withoutSemicolon + ");";
  };
  createSandbox = function(ddoc, propPath, logEntries, overrides) {
    var fun, name, sb;
    sb = {
      log: function(msg) {
        return logEntries.push(msg);
      },
      require: makeRequireFun(makeRefStack(ddoc, propPath))
    };
    for (name in overrides) {
      fun = overrides[name];
      sb[name] = fun;
    }
    return sb;
  };
  makeRefStack = function(ddoc, funPath) {
    var stack;
    stack = [ddoc];
    funPath.split('.').forEach(function(prop) {
      var currObj;
      currObj = stack[stack.length - 1];
      return stack.push(currObj[prop]);
    });
    stack.pop();
    return stack;
  };
  makeRequireFun = function(refStack) {
    return function(moduleID) {
      var code, nextRefStack, sandbox, _ref2;
      sandbox = {
        exports: {}
      };
      try {
        _ref2 = findModule(moduleID, refStack), nextRefStack = _ref2[0], code = _ref2[1];
        sandbox.require = makeRequireFun(nextRefStack);
        runInNewContext(code, sandbox, moduleID);
      } catch (e) {
        throw "Design doc require() error: " + e;
      }
      return sandbox.exports;
    };
  };
  findModule = function(moduleID, refStack) {
    var code, current, next, nextRefStack, part, _i, _len, _ref2;
    code = null;
    nextRefStack = /^\.\.?\//.test(moduleID) ? refStack.slice() : refStack.slice(0, 1);
    _ref2 = moduleID.split('/');
    for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
      part = _ref2[_i];
      if (part === '..') {
        if (nextRefStack.length > 1) {
          nextRefStack.pop();
        }
      } else if (part !== '.') {
        current = nextRefStack[nextRefStack.length - 1];
        next = current[part];
        if (next == null) {
          throw "Couldn't find part '" + part + "' of '" + moduleID + "'";
        }
        nextRefStack.push(next);
      }
    }
    code = nextRefStack.pop();
    return [nextRefStack, code];
  };
  ViewMapFunction = (function() {
    __extends(ViewMapFunction, CouchFunction);
    function ViewMapFunction(ddoc, viewName) {
      var emitted;
      this.ddoc = ddoc;
      this.viewName = viewName;
      this.emitted = emitted = [];
      ViewMapFunction.__super__.constructor.call(this, this.ddoc, "views." + viewName + ".map", {
        emit: function(k, v) {
          return emitted.push({
            key: k,
            value: v
          });
        }
      });
    }
    return ViewMapFunction;
  })();
  ViewReduceFunction = (function() {
    __extends(ViewReduceFunction, CouchFunction);
    function ViewReduceFunction(ddoc, viewName) {
      this.ddoc = ddoc;
      this.viewName = viewName;
      ViewReduceFunction.__super__.constructor.call(this, this.ddoc, "views." + viewName + ".reduce", {
        sum: function(values) {
          return values.reduce((function(total, n) {
            return total + n;
          }), 0);
        }
      });
    }
    return ViewReduceFunction;
  })();
  UpdateFunction = (function() {
    __extends(UpdateFunction, CouchFunction);
    function UpdateFunction(ddoc, updateName, docID) {
      this.ddoc = ddoc;
      this.updateName = updateName;
      this.docID = docID;
      UpdateFunction.__super__.constructor.call(this, this.ddoc, "updates." + updateName);
    }
    UpdateFunction.prototype.call = function(doc, req) {
      if (doc == null) {
        doc = {};
      }
      if (req == null) {
        req = {};
      }
      if (this.docID != null) {
        req.id = this.docID;
      }
      return UpdateFunction.__super__.call.call(this, doc, req);
    };
    return UpdateFunction;
  })();
  FilterFunction = (function() {
    __extends(FilterFunction, CouchFunction);
    function FilterFunction(ddoc, updateName) {
      this.ddoc = ddoc;
      this.updateName = updateName;
      FilterFunction.__super__.constructor.call(this, this.ddoc, "filters." + updateName);
    }
    return FilterFunction;
  })();
  MissingPropPathError = exports.MissingPropPathError = (function() {
    __extends(MissingPropPathError, Error);
    function MissingPropPathError(funPath, ddoc) {
      var _ref2;
      this.funPath = funPath;
      this.ddoc = ddoc;
      this.message = "Property path '" + funPath + "' not found in design doc '" + ((_ref2 = this.ddoc) != null ? _ref2._id : void 0) + "'";
    }
    return MissingPropPathError;
  })();
  NotAFunctionError = exports.NotAFunctionError = (function() {
    __extends(NotAFunctionError, Error);
    function NotAFunctionError(funPath, ddoc) {
      var _ref2;
      this.funPath = funPath;
      this.ddoc = ddoc;
      this.message = "Property path '" + funPath + "' in design doc '" + ((_ref2 = this.ddoc) != null ? _ref2._id : void 0) + "' does not evaluate to a function";
    }
    return NotAFunctionError;
  })();
}).call(this);
